<?php

/**
 * @file
 * Contains \Drupal\dbee\Form\DbeeAdminForm.
 */

namespace Drupal\dbee\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Element;

class DbeeAdminForm extends FormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'dbee_admin_form';
  }

  public function buildForm(array $form, \Drupal\Core\Form\FormStateInterface $form_state) {
    $form['dbee'] = [
      '#type' => 'fieldset',
      '#title' => t('DataBase Email Encryption module'),
    ];

    $description = t('Enable this feature only if you meet performance troubles on the user search.');
    $description .= '<br>' . t('Thoses performance troubles may occures on one specific situation :');

    $conditions = array();
    $conditions[] = t('<a href="@url">Searching a user</a>', array('@url' => \Drupal\Core\Url::fromRoute('search.view_user_search')->toString()));
    $conditions[] = t("by a connected user granted with the 'administer users' permission", array('@url' => \Drupal\Core\Url::fromRoute('search.view_user_search')->toString()));
    $conditions[] = t('on a site with a lot of register users.');
    // @TODO : display as items. (see https://www.drupal.org/node/2195739)
    // $description .= theme('item_list', array('items' => $conditions));
    $description .= implode(', ', $conditions);

    $description .= t("As a limitation, only completed emails will be finded. As an example, on site storing a user with the 'john@example.com' mail value, if this checkbox is checked, searching this user using the key word 'hn@examp' will fail but the whole email address search key 'john@example.com' will succeed. In both cases, the result will be returned very fast.");
    $form['dbee']['custom_search'] = [
      '#type' => 'checkbox',
      '#title' => t('Disable searching users via uncompleted email address.'),
      '#default_value' => (\Drupal::config('dbee.settings')->get('dbee_custom_search') == 1),
      '#description' => $description,
      //'#suffix' => $ui,
    ];

    // Load button.
    $form['actions'] = ['#type' => 'actions'];

    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => t('Save'),
    ];

    return $form;
  }

  public function submitForm(array &$form, \Drupal\Core\Form\FormStateInterface $form_state) {
    if ($form_state->getValue(['custom_search']) != \Drupal::config('dbee.settings')->get('dbee_custom_search')) {
      \Drupal::configFactory()->getEditable('dbee.settings')->set('dbee_custom_search', $form_state->getValue(['custom_search']))->save();
      drupal_set_message(t('New setting has been saved'));
    }
  }
}
