<?php
namespace Drupal\dbee\Tests;

/**
 * Sending/receiving emails with the Contact module.
 *
 * Verify if sending mail via the contact form is not altered.
 *
 * @group dbee
 */
class DbeeContactMailTestCase extends DbeeWebTestBase {
  protected $sender_user;
  protected $recipient_user;

  /**
   * Modules to enable Do not enable the dbee module yet.
   *
   * @var array
   */
  public static $modules = array('contact', 'dbee');

  public function setUp() {
    // Enable any modules required for the test.
    parent::setUp();
    $this->recipient_user = $this->drupalCreateUser();
    $this->sender_user = $this->drupalCreateUser(array('access user contact forms'));
  }

  protected function testContactMail() {
    $this->drupalLogin($this->sender_user);
    $contact_uri = "user/{$this->recipient_user->id()}/contact";
    $this->drupalGet($contact_uri);
    $this->assertText($this->sender_user->mail->value, "Sender's email address is available on the contact form page.");
    $edit = array(
      'subject[0][value]' => $this->randomString(20),
      'message[0][value]' => $this->randomString(128),
    );
    $this->drupalPostForm($contact_uri, $edit, t('Send message'));
    $this->assertRaw(t('Your message has been sent.'), t('The email has been sent.'));
    $this->assertMail('to', $this->recipient_user->mail->value, 'The recipient email address is valid');
    $this->assertMail('reply-to', $this->sender_user->mail->value, 'The sender email address is valid');
  }
}
