<?php
namespace Drupal\dbee\Tests;

/**
 * Check : loading user by id, by email, loading current user, saving user.
 *
 * Verify if the user_save(), user_load(), user_load_by_mail() core functions
 * are not altered by the dbee module.
 *
 * @group dbee
 */
class DbeeCoreFunctionsTestCase extends DbeeWebTestBase {

  protected $user_to_load;
  protected $user_save_user;
  protected $connected_user;
  /**
   * User with permission to enable/uninstall modules.
   *
   * @var array
   */
  protected $admin_modules_account;

  public static $modules = array('aes', 'user');

  public function setUp() {
    // Enable any modules required for the test.
    parent::setUp();
    $this->user_to_load = $this->drupalCreateUser();
    // drupalCreateUser() set an empty 'init' value. Fix it.
    $this->user_to_load->init->value = $this->randomMachineName() . '@eXample.com';
    $this->user_to_load->save();

    // Create a user who can enable the dbee module.
    $this->admin_modules_account = $this->drupalCreateUser(array('administer modules'));
  }

  protected function testCoreFunctions() {
    // Test the user_save(), user_load(), user_load_by_mail() core functions.

    $this->drupalLogin($this->user_to_load);

    $mail_to_load = $this->user_to_load->mail->value;
    // Store a loaded user from mail without the dbee module.

    // Current user.
    $connected_user_mail_original = \Drupal::currentUser()->getEmail();

    // Install dbee module.
    $this->drupalLogin($this->admin_modules_account);
    $edit3 = array('modules[Other][dbee][enable]' => TRUE);
    $this->drupalPostForm('admin/modules', $edit3, t('Install'));
    $this->rebuildContainer();
    $this->assertTrue($this->container->get('module_handler')->moduleExists('dbee'), 'dbee module installed.');
    // Current user.
    $this->drupalLogout();
    $this->drupalLogin($this->user_to_load);

    // Test the user_load().
    $user_load_dbee = \Drupal::entityManager()->getStorage('user')->load($this->user_to_load->id());
    $this->assertEqual($user_load_dbee->mail->value, $mail_to_load, 'On loading a user, its email is avalaible.');


    // Test the user_load_by_mail().
    $user_load_by_mail_dbee = user_load_by_mail($mail_to_load);
    $mail_to_load_dbee = (!empty($user_load_by_mail_dbee)) ? $user_load_by_mail_dbee->mail->value : FALSE;
    $this->assertEqual($mail_to_load_dbee, $mail_to_load, 'the user_load_by_mail() fonction is not altered by the dbee module');

    $connected_user_mail_dbee = \Drupal::currentUser()->getEmail();
    $this->assertEqual($connected_user_mail_dbee, $connected_user_mail_original, 'the connected user is not altered by the dbee module');

    // Save user
    $user_load_dbee->save();
    $saved_mail = (!empty($user_load_dbee)) ? $user_load_dbee->mail->value : FALSE;
    $this->assertEqual($saved_mail, $mail_to_load, 'After saving a user : its email is available.');
  }
}
