<?php
namespace Drupal\dbee\Tests;

/**
 * Edit users.
 *
 * Test the user mail edition.
 *
 * @group dbee
 */
class DbeeEditUserTestCase extends DbeeWebTestBase {
  protected $edited_user_account;
  protected $existing_user;
  public static $modules = array('dbee');

  public function setUp() {
    parent::setUp();
    // Create a basic user with mail = 'example@example.com'. This email will
    // be used to testing if the system prevent from creating a new user with
    // an existing email.
    // Create a user, with sensitive case mail.
    $this->existing_user = $this->drupalCreateUser(array());

    $this->edited_user_account = $this->drupalCreateUser(array());
    // drupalCreateUser() set an empty 'init' value. Fix it.
    $this->edited_user_account->init->value = $this->randomMachineName() . '@example.com';
    $this->edited_user_account->save();

  }

  public function testEditUser() {

    $this->drupalLogin($this->edited_user_account);
    $uid = $this->edited_user_account->id();
    $pass = $this->edited_user_account->pass_raw;

    $data0 = array(
      $uid => array(
        'mail' => $this->edited_user_account->mail->value,
        'init' => $this->edited_user_account->init->value,
      ),
    );
    $this->assertTrue($this->dbee_all_users_valid($data0), 'The user is correctly encrypted.');
    // Check if the email is decrypted on the user edit page.
    $this->drupalGet('user/' . $uid . '/edit');
    $this->assertFieldById('edit-mail', $data0[$uid]['mail'], 'The email on the user edit page is available.');

    // Set the email to lowercase.
    $edit1 = array(
      'mail' => \Drupal\Component\Utility\Unicode::strtolower($data0[$uid]['mail']),
      'current_pass' => $pass,
    );
    $this->drupalPostForm('user/' . $uid . '/edit', $edit1, t('Save'));
    $data1 = $data0;
    $data1[$uid]['mail'] = $edit1['mail'];
    $this->assertTrue($this->dbee_all_users_valid($data1), 'The user mail is correctly encrypted.');

    // Check is email is still encryped after mail edit, Check case sensitive.
    // Email address :
    $edit3 = array(
      'mail' => $this->randomMachineName() . '@EXAMple.com',
      'current_pass' => $pass,
    );
    $this->drupalPostForm('user/' . $uid . '/edit', $edit3, t('Save'));
    // Check user account on edition.
    $this->assertText(t('The changes have been saved.'), 'We successfully edit the user email address : to a new case sensitive email address :');
    // Check if the stored email is encrypted.
    $data3 = $data1;
    $data3[$uid]['mail'] = $edit3['mail'];
    $this->assertTrue($this->dbee_all_users_valid($data3), 'The user mail is correctly encrypted.');
    // Check if the email is decrypted on the user edit page.
    $this->drupalGet('user/' . $uid . '/edit');
    $this->assertFieldById('edit-mail', $edit3['mail'], 'email on the user edit page is still available AND case sensitive is respected.');

    // Check is email is still encryped after mail edit, back to a new
    // lowercase email address :
    $edit4 = array(
      'mail' => \Drupal\Component\Utility\Unicode::strtolower($this->randomMachineName() . '@example.com'),
      'current_pass' => $pass,
    );
    $this->drupalPostForm('user/' . $uid . '/edit', $edit4, t('Save'));
    // Check user account on edition.
    $this->assertText(t('The changes have been saved.'), 'We successfully edit the user email address again : back to a new lower case email address :');
    // Check if the stored email is encrypted.
    $data4 = $data3;
    $data4[$uid]['mail'] = $edit4['mail'];
    $this->assertTrue($this->dbee_all_users_valid($data4), 'The user mail  is correctly encrypted.');
    // Check if the email is decrypted on the user edit page.
    $this->drupalGet('user/' . $uid . '/edit');
    $this->assertFieldById('edit-mail', $edit4['mail'], 'The email on the user edit page is still available.');

    // Try to change the email address to an existing one from another user. The
    // system should not validate it.
    $edit5 = array(
      'mail' => \Drupal\Component\Utility\Unicode::strtoupper($this->existing_user->mail->value),
      'current_pass' => $pass,
    );
    $this->drupalPostForm('user/' . $uid . '/edit', $edit5, t('Save'));
    // Check if new user account has not been created.
    // From the user_account_form_validate() function.
    $this->assertRaw(t('The email address %email is already taken.', array('%email' => $edit5['mail'])), 'The system successfully detects when someone trying to save the same email twice, even if there is case conflict.');
    // Check if the stored email is encrypted.
    $this->assertTrue($this->dbee_all_users_valid($data4), 'The user informations have not changed');
  }
}
