<?php
namespace Drupal\dbee\Tests;


/**
 * Database queries on user emails.
 *
 *  May be used by contrib modules.
 *
 * @group dbee
 */
class DbeeQueryUsersTestCase extends DbeeWebTestBase {
  protected $user_2;
  protected $user_3;
  protected $user_4;
  protected $query_results;
  protected $user_view_users;
  /**
   * User with permission to enable/uninstall modules.
   *
   * @var array
   */
  protected $admin_modules_account;

  public static $modules = array('user', 'aes', 'dbee');//, 'user', 'aes');

  public function setUp() {
    // Enable any modules required for the test.
    parent::setUp();
    // Create a user who can enable the dbee module.
    $this->admin_modules_account = $this->drupalCreateUser(array('administer modules'));
    $this->user_view_users = $this->drupalCreateUser(array('access user profiles'));

    $this->user_2 = $this->drupalCreateUser();
    $this->user_3 = $this->drupalCreateUser();
    $this->user_4 = $this->drupalCreateUser();

    // drupalCreateUser() set an empty 'init' value. Fix it.
    $this->user_2->init->value = $this->randomMachineName() . '@example.com';
    $this->user_2->save();
    $this->user_3->init->value = 'an_invalid_email';
    $this->user_3->save();
    $this->user_4->init->value = $this->randomMachineName() . '@example.com';
    $this->user_4->save();
  }

  protected function testQueryFunctions() {

    $test_title = $verbose = array();
    foreach (array(1, 0) as $dbee) {

      if (!$dbee) {
        // uninstall the dbee module.
        $this->drupalLogin($this->admin_modules_account);
        $edit1 = array('uninstall[dbee]' => TRUE);
        $this->drupalPostForm('admin/modules/uninstall', $edit1, t('Uninstall'));
        // Confirmation page.
        $this->drupalPostForm('admin/modules/uninstall/confirm', array(), t('Uninstall'));
        $this->rebuildContainer();
        $this->assertFalse($this->container->get('module_handler')->moduleExists('dbee'), 'dbee module uninstalled.');

      }

      $this->drupalLogin($this->user_view_users);
      foreach (array('mail', 'init') as $dbee_field) {
        $result_uids = array();
        for ($test = 1; $test <= 14; $test++) {

          $result_uids = array();
          switch ($test) {
            case 1:
              // Look for empty sting.
              $test_title[$test] = 'querying empty mail or init value, operator \'=\'.';
              $keys = '';
              $operator = '=';
              break;

            case 2:
              // Look for empty sting.
              $test_title[$test] = 'querying empty mail or init value, operator \'<>\'.';
              // $keys same than test 1.
              $operator = '<>';
              break;

            case 3:
              // Test searching 1 whole email.
              $test_title[$test] = 'querying a user by mail or init exact value, operator \'=\'.';
              $keys = $this->user_2->$dbee_field->value;
              $operator = '=';
              break;

            case 4:
              // Test searching 1 whole email.
              $test_title[$test] = 'querying a user by mail or init exact value, operator \'<>\'.';
              // $keys same than test 3.
              $operator = '<>';
              break;

            case 5:
              // Test searching 1 whole mail email value changing case.
              $test_title[$test] = 'querying a user by mail or init with sensitive case conflict value, operator \'=\'.';
              $keys = \Drupal\Component\Utility\Unicode::strtoupper($this->user_2->$dbee_field->value);
              $operator = '=';
              break;

            case 6:
              // Test searching 1 whole mail email value changing case.
              $test_title[$test] = 'querying a user by mail or init with sensitive case conflict value, operator \'<>\'.';
              // $keys same than test 5.
              $operator = '<>';
              break;

            case 7:
              // Test searching an mail email value changing case with 'like' operator.
              // $keys is the same as test 5 and 6.
              $test_title[$test] = 'querying a user by mail or init with sensitive case conflict value, operator \'LIKE\'.';
              $operator = 'LIKE';
              break;

            case 8:
              // Test searching an mail email value changing case with 'like' operator.
              // $keys is the same as test 5, 6 and 7.
              $test_title[$test] = 'querying a user by mail or init with sensitive case conflict value, operator \'NOT LIKE\'.';
              $operator = 'NOT LIKE';
              break;

            case 9:
              // Test searching 2 whole email.
              $test_title[$test] = 'querying 2 user by mail or init exact value, operator \'IN\'.';
              $keys = array(
                $this->user_2->$dbee_field->value,
                $this->user_3->$dbee_field->value,
              );
              $operator = 'IN';
              break;

            case 10:
              // Test searching 2 whole email.
              // $keys is the same than test 9.
              $test_title[$test] = 'querying 2 user by mail or init exact value, operator \'NOT IN\'.';
              $operator = 'NOT IN';
              break;

            case 11:
              // Test searching 3 users  by a partial key.
              $keys = '%exAM_le.com%';
              $test_title[$test] = 'querying users by mail or init value using wildcard (\'' . $keys . '\'), operator \'LIKE\'.';
              $operator = 'LIKE';
              break;

            case 12:
              // Test searching 3 users  by a partial key.
              // $keys is the same than test 11.
              $test_title[$test] = 'querying users by mail or init value using wildcard (\'' . $keys . '\'), operator \'NOT LIKE\'.';
              $operator = 'NOT LIKE';
              break;

            case 13:
              // Using nested conditions.
              $test_title[$test] = 'querying users by mail or init value using wildcard  (\'' . $keys . '\') on nested conditions, operator \'LIKE\'.';
              // $keys is the same than test 11 and 12.
              $operator = 'LIKE';
              break;

            case 14:
              // Using nested conditions.
              $test_title[$test] = 'querying users by mail or init value using wildcard  (\'' . $keys . '\') on nested conditions, operator \'NOT LIKE\'.';
              // $keys is the same than test 11, 12 and 13.
              $operator = 'NOT LIKE';
              break;

          }

          $query = db_select('users_field_data', 'u');
          $query->fields('u', array('uid', 'mail', 'name', 'status', 'created', 'access'));

          if ($test < 13) {
            $query->condition("u.$dbee_field", $keys, $operator);
          }
          else {
            $query->condition(db_or()->
            condition('name', 'Not exists', '=')->
            condition($dbee_field, $keys, $operator));
          }
          // The hook_query_alter function only works on dynamic tagged queries.
          $query->addTag('test');
          $query->orderBy('uid');
          $result = $query->execute();
          $result_uids = array();
          foreach ($result as $record) {
            $result_uids[] = $record->uid;
          }
          // Save the test result.
          $query_args = $query->arguments();
          $this->query_results[$test][$dbee][$dbee_field] = $result_uids;
          $verbose[$test][$dbee][$dbee_field] = "test #{$test}, dbee " . (($dbee) ? 'enabled' : 'uninstalled') . ", field : {$dbee_field}" .
            (($dbee) ? '' : (($this->query_results[$test][0][$dbee_field] == $this->query_results[$test][1][$dbee_field]) ? '<br />Test succeed' : '<br /><strong>Test Failed</strong>')) .
            '<br />' . (($dbee) ? 'Transformed query' : 'Query') . ' --> ' . strtr($query->__toString(), $query_args) .
            '<br /> ===> returned users ids : ' . implode(' ,', $result_uids);
        }
      }
    }

    // Explore results.
    foreach ($this->query_results as $test => $data) {
      foreach (array('mail', 'init') as $dbee_field) {
        $this->verbose($verbose[$test][0][$dbee_field] . '<br /><br />' . $verbose[$test][1][$dbee_field]);
      }
      $this->assertEqual($data[0], $data[1], "test #{$test} succeed : {$test_title[$test]}");
    }
  }
}
