<?php
namespace Drupal\dbee\Tests;

use Drupal\Core\Link;

/**
 * Search users by email
 *
 * Ensure searching user via email address is still available. Try first without
 * the dbee module, then with the dbee module enabled.
 *
 * @group dbee
 */
class DbeeSearchUserTestCase extends DbeeWebTestBase {
  protected $existing_user1;
  protected $existing_user2;
  protected $existing_user3;
  protected $provider = 'MYprovider.com';
  protected $search_users_account;
  protected $admin_modules_account;

  public static $modules = array('user', 'aes', 'search');

  public function setUp() {
    // Enable any modules required for the test.
    parent::setUp();

    // Create a user, with sensitive case mail.
    $this->existing_user1 = $this->drupalCreateUser();
    // Make sure the email and name are distincts.
    $this->existing_user1->name->value = 'anyname1';
    $this->existing_user1->mail->value = $this->randomMachineName() . '@' . $this->provider;
    $this->existing_user1->save();

    $this->existing_user2 = $this->drupalCreateUser();
    // Make sure the email and name are distincts.
    $this->existing_user2->name->value = 'anyname2';
    $this->existing_user2->mail->value = $this->randomMachineName() . '@' . $this->provider;
    $this->existing_user2->save();

    // Extra user, do nothing.
    $this->existing_user3 = $this->drupalCreateUser();

    // Create a user who can search by email address and log in.
    $this->search_users_account = $this->drupalCreateUser(array(
      'administer users',
      'search content',
      'access user profiles',
    ));
    // Create a user who can enable the dbee module.
    $this->admin_modules_account = $this->drupalCreateUser(array('administer modules'));
  }

  protected function testSearchUser() {
    // Test the feature : search user using an existing email address : first
    // without the dbee module, then with dbee module enable.

    $this->drupalLogin($this->search_users_account);
    // Search the completed email address.
    $edit1 = array('keys' => \Drupal\Component\Utility\Unicode::strtoupper($this->existing_user1->mail->value));
    $mail1 = $edit1['keys'];
    $this->drupalPostForm('search/user', $edit1, t('Search'));
    $link1 = Link::fromTextAndUrl($this->existing_user1->name->value . ' (' . $this->existing_user1->mail->value . ')', \Drupal\Core\Url::fromRoute('entity.user.canonical', ['user' => $this->existing_user1->id()], array('absolute' => TRUE)))->toString();
    $this->assertRaw($link1, 'searching user by completed email address succeed');

    // Search the partial email address.
    $edit2 = array('keys' => \Drupal\Component\Utility\Unicode::strtoupper($this->provider));
    $this->drupalPostForm('search/user', $edit2, t('Search'));
    $link2 = Link::fromTextAndUrl($this->existing_user1->name->value . ' (' . $this->existing_user1->mail->value . ')', \Drupal\Core\Url::fromRoute('entity.user.canonical', ['user' => $this->existing_user1->id()], array('absolute' => TRUE)))->toString();
    $this->assertRaw($link2, 'searching user by partial email address succeed (first user finded)');

    $link3 = Link::fromTextAndUrl($this->existing_user2->name->value . ' (' . $this->existing_user2->mail->value . ')', \Drupal\Core\Url::fromRoute('entity.user.canonical', ['user' => $this->existing_user2->id()], array('absolute' => TRUE)))->toString();
    $this->assertRaw($link3, 'searching user by partial email address succeed (second user finded)');

    $link4 = Link::fromTextAndUrl($this->existing_user3->name->value . ' (' . $this->existing_user3->mail->value . ')', \Drupal\Core\Url::fromRoute('entity.user.canonical', ['user' => $this->existing_user3->id()], array('absolute' => TRUE)))->toString();
    $this->assertNoRaw($link4, 'searching user by partial email address succeed (third user not finded)');

    // Install dbee module.
    $this->drupalLogin($this->admin_modules_account);
    $edit3 = array('modules[Other][dbee][enable]' => TRUE);
    $this->drupalPostForm('admin/modules', $edit3, t('Install'));
    $this->rebuildContainer();
    $this->assertTrue($this->container->get('module_handler')->moduleExists('dbee'), 'dbee module installed.');

    // Search again.
    $this->drupalLogin($this->search_users_account);

    $this->drupalPostForm('search/user', array('keys' => $mail1), t('Search'));
    $this->assertRaw($link1, 'searching user by completed email address still succeed');

    // Search the partial email address.
    $this->drupalPostForm('search/user', $edit2, t('Search'));
    $this->assertRaw($link2, 'searching user by partial email address still succeed (first user finded)');
    $this->assertRaw($link3, 'searching user by partial email address still succeed (second user finded)');
    $this->assertNoRaw($link4, 'searching user by partial email address still succeed (third user not finded)');
  }
}
