<?php
namespace Drupal\dbee\Tests;

/**
 * User reset password from email.
 *
 * Test sending email for lost password user.
 *
 * @group dbee
 */
class DbeeUserResetPassTestCase extends DbeeWebTestBase {
  protected $edited_user_account;
  protected $existing_user;
  /**
   * Modules to enable. Here do not enabled dbee module yet.
   *
   * @var array
   */
  public static $modules = array('dbee');


  public function setUp() {
    parent::setUp();
    // Create a basic user with mail = 'example@example.com'. This email will
    // be used to testing if the system prevent from creating a new user with
    // an existing email.
    // Create a user, with sensitive case mail.
    $this->existing_user = $this->drupalCreateUser();

    $this->edited_user_account = $this->drupalCreateUser();
    // Make sure the mail is sensitive case.
    $this->edited_user_account->mail->value = $this->edited_user_account->name->value . '@eXample.com';
    // drupalCreateUser() set an empty 'init' value. Fix it.
    $this->edited_user_account->init->value = $this->randomMachineName() . '@example.com';
    $this->edited_user_account->save();
    // Make sure we are logged out.
  }

  public function testUserPass() {

    $uid = $this->edited_user_account->id();

    $data0 = array(
      $uid => array(
        'mail' => $this->edited_user_account->mail->value,
        'init' => $this->edited_user_account->init->value,
      ),
    );
    $this->assertTrue($this->dbee_all_users_valid($data0), 'The user is encrypted and can be decrypted back');
    // Go to the lost password page.
    $this->drupalGet('user/password');
    // Set the email sensitive case.
    $edit1 = array(
      'name' => $data0[$uid]['mail'],
    );
    $this->drupalPostForm('user/password', $edit1, t('Submit'));
    $this->assertText(t('Further instructions have been sent to your email address.'), 'login link successfully sent for the exact sensitive case email');

    $this->drupalGet('user/password');
    // Set the email to lowercase.
    $edit2 = array(
      'name' => \Drupal\Component\Utility\Unicode::strtolower($data0[$uid]['mail']),
    );
    $this->drupalPostForm('user/password', $edit2, t('Submit'));
    $this->assertText(t('Further instructions have been sent to your email address.'), 'login link successfully sent for lowercase email');

    // Set the email to uppercase.
    $edit3 = array(
      'name' => \Drupal\Component\Utility\Unicode::strtoupper($data0[$uid]['mail']),
    );
    $this->drupalPostForm('user/password', $edit3, t('Submit'));
    $this->assertText(t('Further instructions have been sent to your email address.'), 'login link successfully sent for other sensitive case email');

    $edit4 = array(
      'name' => $this->edited_user_account->name->value,
    );
    $this->drupalPostForm('user/password', $edit4, t('Submit'));
    $this->assertText(t('Further instructions have been sent to your email address.'), 'login link successfully sent for username');
  }
}
